/**********************************************************************
 Freeciv - Copyright (C) 2002 - Per I. Mathisen
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "shared.h"
#include "support.h"

#include "capability.h"
#include "capstr.h"
#include "city.h"
#include "events.h"
#include "fcintl.h"
#include "game.h"
#include "log.h"
#include "map.h"
#include "mem.h"
#include "nation.h"
#include "netintf.h"
#include "packets.h"
#include "player.h"
#include "rand.h"
#include "registry.h"
#include "tech.h"
#include "timing.h"
#include "version.h"
#include "autoattack.h"
#include "barbarian.h"
#include "cityhand.h"
#include "citytools.h"
#include "cityturn.h"
#include "console.h"
#include "diplhand.h"
#include "gamehand.h"
#include "gamelog.h"
#include "handchat.h"
#include "mapgen.h"
#include "maphand.h"
#include "meta.h"
#include "plrhand.h"
#include "report.h"
#include "ruleset.h"
#include "sanitycheck.h"
#include "savegame.h"
#include "sernet.h"
#include "settlers.h"
#include "spacerace.h"
#include "stdinhand.h"
#include "unithand.h"
#include "unittools.h"
#include "advmilitary.h"
#include "aihand.h"

#include "tests.h"
#include "srv_main.h"
#include "diplomats.h"

/* command-line arguments to server */
struct server_arguments srvarg;

int main(int argc, char **argv)
{
  struct section_file file;

  /* initialize server */
  srv_init();

  con_log_init(srvarg.log_filename, srvarg.loglevel);
  gamelog_init(srvarg.gamelog_filename);
  gamelog_set_level(GAMELOG_FULL);
  gamelog(GAMELOG_NORMAL, _("Starting new log"));

  init_our_capability();
  game_init();
  srvarg.load_filename = "standard.sav.gz";

  if(!section_file_load_nodup(&file, srvarg.load_filename)) {
    freelog(LOG_FATAL, _("Couldn't load savefile: %s"), srvarg.load_filename);
    exit(EXIT_FAILURE);
  }
  game_load(&file);
  section_file_check_unused(&file, srvarg.load_filename);
  section_file_free(&file);

  server_state=PRE_GAME_STATE;

  if (game.auto_ai_toggle) {
    players_iterate(pplayer) {
      if (!pplayer->is_connected && !pplayer->ai.control) {
        toggle_ai_player_direct(NULL, pplayer);
      }
    } players_iterate_end;
  }

  server_state = RUN_GAME_STATE;

  initialize_move_costs(); /* this may be the wrong place to do this */
  generate_minimap(); /* for city_desire; saves a lot of calculations */

  if (!game.is_new_game) {
    players_iterate(pplayer) {
      civ_score(pplayer);       /* if we don't, the AI gets really confused */
      if (pplayer->ai.control) {
        set_ai_level_direct(pplayer, pplayer->ai.skill_level);
        assess_danger_player(pplayer); /* a slowdown, but a necessary one */
      }
    } players_iterate_end;
  }
  game.is_new_game = FALSE;

  {
  struct unit *punit = find_unit_by_id(3); /* diplomat */
  FC_TEST_START("COMMON");
  FC_TEST(diplomat_can_do_action(punit, DIPLOMAT_BRIBE, 39, 5), "Diplomat bribe unit");
  FC_TEST(!diplomat_can_do_action(punit, DIPLOMAT_BRIBE, 37, 5), "!Diplomat bribe unitless tile");
  punit = find_unit_by_id(4); /* spy */
  FC_TEST(diplomat_can_do_action(punit, DIPLOMAT_BRIBE, 39, 1), "Spy bribe unit");
  FC_TEST(diplomat_can_do_action(punit, DIPLOMAT_EMBASSY, 37, 1), "Spy embassy city");
  FC_TEST(diplomat_can_do_action(punit, DIPLOMAT_STEAL, 37, 1), "Spy steal tech from city");
  FC_TEST(diplomat_can_do_action(punit, DIPLOMAT_INVESTIGATE, 37, 1), "Spy investigate city");
  FC_TEST(is_diplomat_unit(punit), "Spy is diplomat (sic)");
  punit = find_unit_by_id(9); /* stealth fighter */
  FC_TEST(is_air_unit(punit), "Fighter is air unit");
  FC_TEST(!is_heli_unit(punit), "!Fighter is heli unit");
  FC_TEST(!is_ground_unit(punit), "!Fighter is ground unit");
  punit = find_unit_by_id(8); /* helicopter */
  FC_TEST(!is_air_unit(punit), "!Helicopter is air unit");
  FC_TEST(is_heli_unit(punit), "Helicopter is heli unit");
  FC_TEST(!is_ground_unit(punit), "!Helicopter is ground unit");
  }

  {
  struct player *pplayer = find_player_by_name("Wang Gon"); 
  struct city *pcity = city_list_find_id(&pplayer->cities, 16);
  FC_TEST_START("SERVER");
  FC_TEST(count_diplomats_on_tile(38,5) == 1, "Counting diplomats");
  FC_TEST(city_got_barracks(pcity), "City has barracks");
  FC_TEST((find_closest_owned_city(pplayer, 41, 2, FALSE, NULL) == pcity), "Find closest city");
  FC_TEST(city_can_work_tile(pcity, 1, 1), "City can work tile");
  FC_TEST(can_place_worker_here(pcity, 1, 1), "City can place worker at tile");
  }

  return 0;
}
